@@ -37,9 +37,7 @@
 #include <glib-object.h>
 #include <gtk/gtk.h>
 
-#include <libnautilus-extension/nautilus-extension-types.h>
-#include <libnautilus-extension/nautilus-menu-provider.h>
-#include <libnautilus-extension/nautilus-info-provider.h>
+#include <nautilus-extension.h>
 
 #include "g-util.h"
 #include "dropbox-command-client.h"
@@ -49,7 +47,7 @@
 static char *emblems[] = {"dropbox-uptodate", "dropbox-syncing", "dropbox-unsyncable"};
 gchar *DEFAULT_EMBLEM_PATHS[2] = { EMBLEMDIR , NULL };
 
-gboolean dropbox_use_nautilus_submenu_workaround;
+
 gboolean dropbox_use_operation_in_progress_workaround;
 
 static GType dropbox_type = 0;
@@ -551,7 +549,6 @@ static int
 nautilus_dropbox_parse_menu(gchar			**options,
 			    NautilusMenu		*menu,
 			    GString			*old_action_string,
-			    GList			*toret,
 			    NautilusMenuProvider	*provider,
 			    GList			*files)
 {
@@ -587,7 +584,7 @@ nautilus_dropbox_parse_menu(gchar			**options,
       g_string_append(new_action_string, "::");
 
       ret += nautilus_dropbox_parse_menu(suboptions, submenu, new_action_string,
-					 toret, provider, files);
+					 provider, files);
 
       item = nautilus_menu_item_new(new_action_string->str,
 				    item_name, "", NULL);
@@ -630,13 +627,6 @@ nautilus_dropbox_parse_menu(gchar			**options,
 	g_object_set_property (G_OBJECT(item), "sensitive", &sensitive);
       }
 
-      /* taken from nautilus-file-repairer (http://repairer.kldp.net/):
-       * this code is a workaround for a bug of nautilus
-       * See: http://bugzilla.gnome.org/show_bug.cgi?id=508878 */
-      if (dropbox_use_nautilus_submenu_workaround) {
-	toret = g_list_append(toret, item);
-      }
-
       g_object_unref(item);
       g_string_free(new_action_string, TRUE);
       ret++;
@@ -661,7 +651,6 @@ get_file_items_callback(GHashTable *response, gpointer ud)
 
 static GList *
 nautilus_dropbox_get_file_items(NautilusMenuProvider *provider,
-                                GtkWidget            *window,
 				GList                *files)
 {
   /*
@@ -716,17 +705,12 @@ nautilus_dropbox_get_file_items(NautilusMenuProvider *provider,
   NautilusDropbox *cvs = NAUTILUS_DROPBOX(provider);
   dropbox_command_client_request(&(cvs->dc.dcc), (DropboxCommand *) dgc);
 
-  GTimeVal gtv;
-
   /*
    * 4. We have to block until it's done because nautilus expects a reply.  But we will
    * only block for 50 ms for a reply.
    */
 
-  g_get_current_time(&gtv);
-  g_time_val_add(&gtv, 50000);
-
-  GHashTable *context_options_response = g_async_queue_timed_pop(reply_queue, &gtv);
+  GHashTable *context_options_response = g_async_queue_timeout_pop(reply_queue, 50000);
   g_async_queue_unref(reply_queue);
 
   if (!context_options_response) {
@@ -744,21 +728,32 @@ nautilus_dropbox_get_file_items(NautilusMenuProvider *provider,
     /* build the menu */
     NautilusMenuItem *root_item;
     NautilusMenu *root_menu;
+    GList *submenu_items;
+    guint list_length;
 
     root_menu = nautilus_menu_new();
-    root_item = nautilus_menu_item_new("NautilusDropbox::root_item",
-				       "Dropbox", "Dropbox Options", "dropbox");
 
-    toret = g_list_append(toret, root_item);
     GString *action_string = g_string_new("NautilusDropbox::");
 
-    if (!nautilus_dropbox_parse_menu(options, root_menu, action_string,
-				     toret, provider, files)) {
-	g_object_unref(toret);
-	toret = NULL;
-    }
+    nautilus_dropbox_parse_menu(options, root_menu, action_string,
+				provider, files);
 
-    nautilus_menu_item_set_submenu(root_item, root_menu);
+    submenu_items = nautilus_menu_get_items(root_menu);
+    list_length = g_list_length(submenu_items);
+
+    if (list_length == 1) {
+      /* Return only the single menu item */
+      toret = submenu_items;
+    } else {
+      /* Construct a Dropbox submenu */
+      root_item = nautilus_menu_item_new("NautilusDropbox::root_item",
+					 "Dropbox", "Dropbox Options",
+					 "dropbox");
+      nautilus_menu_item_set_submenu(root_item, root_menu);
+      toret = g_list_append(toret, root_item);
+
+      g_list_free_full(submenu_items, g_object_unref);
+    }
 
     g_string_free(action_string, TRUE);
     g_object_unref(root_menu);
@@ -778,14 +773,13 @@ add_emblem_paths(GHashTable* emblem_paths_response)
 
   gchar **emblem_paths_list;
   int i;
-
-  GtkIconTheme *theme = gtk_icon_theme_get_default();
+  GtkIconTheme *theme = gtk_icon_theme_get_for_display (gdk_display_get_default ());
 
   if (emblem_paths_response &&
       (emblem_paths_list = g_hash_table_lookup(emblem_paths_response, "path"))) {
       for (i = 0; emblem_paths_list[i] != NULL; i++) {
 	if (emblem_paths_list[i][0])
-	  gtk_icon_theme_append_search_path(theme, emblem_paths_list[i]);
+	  gtk_icon_theme_add_search_path(theme, emblem_paths_list[i]);
       }
   }
   g_hash_table_unref(emblem_paths_response);
@@ -804,15 +798,14 @@ remove_emblem_paths(GHashTable* emblem_paths_response)
       goto exit;
 
   // We need to remove the old paths.
-  GtkIconTheme * icon_theme = gtk_icon_theme_get_default();
   gchar ** paths;
-  gint path_count;
+  GtkIconTheme *theme = gtk_icon_theme_get_for_display (gdk_display_get_default ());
 
-  gtk_icon_theme_get_search_path(icon_theme, &paths, &path_count);
+  paths = gtk_icon_theme_get_search_path(theme);
 
   gint i, j, out = 0;
   gboolean found = FALSE;
-  for (i = 0; i < path_count; i++) {
+  for (i = 0; paths[i] != NULL; i++) {
       gboolean keep = TRUE;
       for (j = 0; emblem_paths_list[j] != NULL; j++) {
 	  if (emblem_paths_list[j][0]) {
@@ -834,7 +827,7 @@ remove_emblem_paths(GHashTable* emblem_paths_response)
      accomodate the changes */
   if (found) {
     paths[out] = NULL; /* Clear the last one */
-    gtk_icon_theme_set_search_path(icon_theme, (const gchar **)paths, out);
+    gtk_icon_theme_set_search_path(theme, (const gchar **)paths);
   }
 
   g_strfreev(paths);
@@ -854,13 +847,13 @@ void get_emblem_paths_cb(GHashTable *emblem_paths_response, NautilusDropbox *cvs
       g_hash_table_ref(emblem_paths_response);
   }
 
-  g_mutex_lock(cvs->emblem_paths_mutex);
+  g_mutex_lock(&(cvs->emblem_paths_mutex));
   if (cvs->emblem_paths) {
     g_idle_add((GSourceFunc) remove_emblem_paths, cvs->emblem_paths);
     cvs->emblem_paths = NULL;
   }
   cvs->emblem_paths = emblem_paths_response;
-  g_mutex_unlock(cvs->emblem_paths_mutex);
+  g_mutex_unlock(&(cvs->emblem_paths_mutex));
 
   g_idle_add((GSourceFunc) add_emblem_paths, g_hash_table_ref(emblem_paths_response));
   g_idle_add((GSourceFunc) reset_all_files, cvs);
@@ -879,22 +872,22 @@ static void
 on_disconnect(NautilusDropbox *cvs) {
   reset_all_files(cvs);
 
-  g_mutex_lock(cvs->emblem_paths_mutex);
+  g_mutex_lock(&(cvs->emblem_paths_mutex));
   /* This call will free the data too. */
   g_idle_add((GSourceFunc) remove_emblem_paths, cvs->emblem_paths);
   cvs->emblem_paths = NULL;
-  g_mutex_unlock(cvs->emblem_paths_mutex);
+  g_mutex_unlock(&(cvs->emblem_paths_mutex));
 }
 
 
 static void
-nautilus_dropbox_menu_provider_iface_init (NautilusMenuProviderIface *iface) {
+nautilus_dropbox_menu_provider_iface_init (NautilusMenuProviderInterface *iface) {
   iface->get_file_items = nautilus_dropbox_get_file_items;
   return;
 }
 
 static void
-nautilus_dropbox_info_provider_iface_init (NautilusInfoProviderIface *iface) {
+nautilus_dropbox_info_provider_iface_init (NautilusInfoProviderInterface *iface) {
   iface->update_file_info = nautilus_dropbox_update_file_info;
   iface->cancel_update = nautilus_dropbox_cancel_update;
   return;
@@ -910,7 +903,7 @@ nautilus_dropbox_instance_init (NautilusDropbox *cvs) {
 					    (GEqualFunc) g_direct_equal,
 					    (GDestroyNotify) NULL,
 					    (GDestroyNotify) g_free);
-  cvs->emblem_paths_mutex = g_mutex_new();
+  g_mutex_init(&(cvs->emblem_paths_mutex));
   cvs->emblem_paths = NULL;
 
   /* setup the connection obj*/
